# -*- coding: utf-8 -*-
# generated file

languages = (
{'ast': {'ast': 'asturianu',
         'bg': 'búlgaru',
         'bs': 'bosniu',
         'cs': 'checu',
         'de': 'alemán',
         'el': 'griegu',
         'en_GB': 'inglés (Reinu Xuníu)',
         'es': 'español',
         'fi': 'finlandés',
         'fr': 'francés',
         'gl': 'gallegu',
         'he': 'hebréu',
         'it': 'italianu',
         'kab': 'kabileñu',
         'ms': 'malayu',
         'pl': 'polacu',
         'pt_BR': 'portugués (Brasil)',
         'ru': 'rusu',
         'uk': 'ucraín',
         'uz': 'uzbequistanín',
         'zh_TW': 'chinu (Taiwán)'},
 'bg': {'ast': 'астурски',
        'bg': 'български',
        'bs': 'босненски',
        'cs': 'чешки',
        'de': 'немски',
        'el': 'гръцки',
        'en_GB': 'английски (Обединеното кралство)',
        'es': 'испански',
        'fi': 'фински',
        'fr': 'френски',
        'gl': 'галисийски',
        'he': 'иврит',
        'it': 'италиански',
        'kab': 'кабилски',
        'ms': 'малайски',
        'pl': 'полски',
        'pt_BR': 'португалски (Бразилия)',
        'ru': 'руски',
        'uk': 'украински',
        'uz': 'узбекски',
        'zh_TW': 'китайски (Тайван)'},
 'bs': {'ast': 'asturijski',
        'bg': 'bugarski',
        'bs': 'bosanski',
        'cs': 'češki',
        'de': 'njemački',
        'el': 'grčki',
        'en_GB': 'engleski (Velika Britanija)',
        'es': 'španski',
        'fi': 'finski',
        'fr': 'francuski',
        'gl': 'galski',
        'he': 'hebrejski',
        'it': 'italijanski',
        'kab': 'kabile',
        'ms': 'malajski',
        'pl': 'poljski',
        'pt_BR': 'portugalski (Brazil)',
        'ru': 'ruski',
        'uk': 'ukrajinski',
        'uz': 'uzbečki',
        'zh_TW': 'kineski (Tajvan)'},
 'cs': {'ast': 'asturština',
        'bg': 'bulharština',
        'bs': 'bosenština',
        'cs': 'čeština',
        'de': 'němčina',
        'el': 'řečtina',
        'en_GB': 'angličtina (Velká Británie)',
        'es': 'španělština',
        'fi': 'finština',
        'fr': 'francouzština',
        'gl': 'galicijština',
        'he': 'hebrejština',
        'it': 'italština',
        'kab': 'kabylština',
        'ms': 'malajština',
        'pl': 'polština',
        'pt_BR': 'portugalština (Brazílie)',
        'ru': 'ruština',
        'uk': 'ukrajinština',
        'uz': 'uzbečtina',
        'zh_TW': 'čínština (Tchaj-wan)'},
 'de': {'ast': 'Asturianisch',
        'bg': 'Bulgarisch',
        'bs': 'Bosnisch',
        'cs': 'Tschechisch',
        'de': 'Deutsch',
        'el': 'Griechisch',
        'en_GB': 'Englisch (Vereinigtes Königreich)',
        'es': 'Spanisch',
        'fi': 'Finnisch',
        'fr': 'Französisch',
        'gl': 'Galizisch',
        'he': 'Hebräisch',
        'it': 'Italienisch',
        'kab': 'Kabylisch',
        'ms': 'Malaiisch',
        'pl': 'Polnisch',
        'pt_BR': 'Portugiesisch (Brasilien)',
        'ru': 'Russisch',
        'uk': 'Ukrainisch',
        'uz': 'Usbekisch',
        'zh_TW': 'Chinesisch (Taiwan)'},
 'el': {'ast': 'Αστουριανά',
        'bg': 'Βουλγαρικά',
        'bs': 'Βοσνιακά',
        'cs': 'Τσεχικά',
        'de': 'Γερμανικά',
        'el': 'Ελληνικά',
        'en_GB': 'Αγγλικά (Ηνωμένο Βασίλειο)',
        'es': 'Ισπανικά',
        'fi': 'Φινλανδικά',
        'fr': 'Γαλλικά',
        'gl': 'Γαλικιανά',
        'he': 'Εβραϊκά',
        'it': 'Ιταλικά',
        'kab': 'Καμπίλε',
        'ms': 'Μαλάι',
        'pl': 'Πολωνικά',
        'pt_BR': 'Πορτογαλικά (Βραζιλία)',
        'ru': 'Ρωσικά',
        'uk': 'Ουκρανικά',
        'uz': 'Ουζμπεκικά',
        'zh_TW': 'Κινεζικά (Ταϊβάν)'},
 'en_GB': {'ast': 'Asturian',
           'bg': 'Bulgarian',
           'bs': 'Bosnian',
           'cs': 'Czech',
           'de': 'German',
           'el': 'Greek',
           'en_GB': 'English (United Kingdom)',
           'es': 'Spanish',
           'fi': 'Finnish',
           'fr': 'French',
           'gl': 'Galician',
           'he': 'Hebrew',
           'it': 'Italian',
           'kab': 'Kabyle',
           'ms': 'Malay',
           'pl': 'Polish',
           'pt_BR': 'Portuguese (Brazil)',
           'ru': 'Russian',
           'uk': 'Ukrainian',
           'uz': 'Uzbek',
           'zh_TW': 'Chinese (Taiwan)'},
 'es': {'ast': 'asturiano',
        'bg': 'búlgaro',
        'bs': 'bosnio',
        'cs': 'checo',
        'de': 'alemán',
        'el': 'griego',
        'en_GB': 'inglés (Reino Unido)',
        'es': 'español',
        'fi': 'finés',
        'fr': 'francés',
        'gl': 'gallego',
        'he': 'hebreo',
        'it': 'italiano',
        'kab': 'cabila',
        'ms': 'malayo',
        'pl': 'polaco',
        'pt_BR': 'portugués (Brasil)',
        'ru': 'ruso',
        'uk': 'ucraniano',
        'uz': 'uzbeko',
        'zh_TW': 'chino (Taiwán)'},
 'fi': {'ast': 'asturia',
        'bg': 'bulgaria',
        'bs': 'bosnia',
        'cs': 'tšekki',
        'de': 'saksa',
        'el': 'kreikka',
        'en_GB': 'englanti (Iso-Britannia)',
        'es': 'espanja',
        'fi': 'suomi',
        'fr': 'ranska',
        'gl': 'galicia',
        'he': 'heprea',
        'it': 'italia',
        'kab': 'kabyyli',
        'ms': 'malaiji',
        'pl': 'puola',
        'pt_BR': 'portugali (Brasilia)',
        'ru': 'venäjä',
        'uk': 'ukraina',
        'uz': 'uzbekki',
        'zh_TW': 'kiina (Taiwan)'},
 'fr': {'ast': 'asturien',
        'bg': 'bulgare',
        'bs': 'bosniaque',
        'cs': 'tchèque',
        'de': 'allemand',
        'el': 'grec',
        'en_GB': 'anglais (Royaume-Uni)',
        'es': 'espagnol',
        'fi': 'finnois',
        'fr': 'français',
        'gl': 'galicien',
        'he': 'hébreu',
        'it': 'italien',
        'kab': 'kabyle',
        'ms': 'malais',
        'pl': 'polonais',
        'pt_BR': 'portugais (Brésil)',
        'ru': 'russe',
        'uk': 'ukrainien',
        'uz': 'ouzbek',
        'zh_TW': 'chinois (Taïwan)'},
 'gl': {'ast': 'asturiano',
        'bg': 'búlgaro',
        'bs': 'bosnio',
        'cs': 'checo',
        'de': 'alemán',
        'el': 'grego',
        'en_GB': 'inglés (Reino Unido)',
        'es': 'español',
        'fi': 'finés',
        'fr': 'francés',
        'gl': 'galego',
        'he': 'hebreo',
        'it': 'italiano',
        'kab': 'kabile',
        'ms': 'malaio',
        'pl': 'polaco',
        'pt_BR': 'portugués (Brasil)',
        'ru': 'ruso',
        'uk': 'ucraíno',
        'uz': 'uzbeco',
        'zh_TW': 'chinés (Taiwán)'},
 'he': {'ast': 'אסטורית',
        'bg': 'בולגרית',
        'bs': 'בוסנית',
        'cs': 'צ׳כית',
        'de': 'גרמנית',
        'el': 'יוונית',
        'en_GB': 'אנגלית (הממלכה המאוחדת)',
        'es': 'ספרדית',
        'fi': 'פינית',
        'fr': 'צרפתית',
        'gl': 'גליציאנית',
        'he': 'עברית',
        'it': 'איטלקית',
        'kab': 'קבילה',
        'ms': 'מלאית',
        'pl': 'פולנית',
        'pt_BR': 'פורטוגלית (ברזיל)',
        'ru': 'רוסית',
        'uk': 'אוקראינית',
        'uz': 'אוזבקית',
        'zh_TW': 'סינית (טייוואן)'},
 'it': {'ast': 'asturiano',
        'bg': 'bulgaro',
        'bs': 'bosniaco',
        'cs': 'ceco',
        'de': 'tedesco',
        'el': 'greco',
        'en_GB': 'inglese (Regno Unito)',
        'es': 'spagnolo',
        'fi': 'finlandese',
        'fr': 'francese',
        'gl': 'galiziano',
        'he': 'ebraico',
        'it': 'italiano',
        'kab': 'cabilo',
        'ms': 'malese',
        'pl': 'polacco',
        'pt_BR': 'portoghese (Brasile)',
        'ru': 'russo',
        'uk': 'ucraino',
        'uz': 'usbeco',
        'zh_TW': 'cinese (Taiwan)'},
 'kab': {'bg': 'Tabulgarit',
         'cs': 'Tačikit',
         'de': 'Talmant',
         'el': 'Tagrikit',
         'en_GB': 'Taglizit (Tagelda Yedduklen)',
         'es': 'Taspenyulit',
         'fr': 'Tafransist',
         'it': 'Taṭalyanit',
         'kab': 'Taqbaylit',
         'ms': 'Tamalawit',
         'pl': 'Tapulunit',
         'pt_BR': 'Tapurtugalit (Brizil)',
         'ru': 'Tarusit',
         'uk': 'Tukranit',
         'zh_TW': 'Tacinwat, Tamundarint (Ṭaywan)'},
 'ms': {'bg': 'Bulgaria',
        'bs': 'Bosnia',
        'cs': 'Czech',
        'de': 'Jerman',
        'el': 'Greek',
        'en_GB': 'Inggeris (United Kingdom)',
        'es': 'Sepanyol',
        'fi': 'Finland',
        'fr': 'Perancis',
        'gl': 'Galicia',
        'he': 'Ibrani',
        'it': 'Itali',
        'kab': 'Kabyle',
        'ms': 'Bahasa Melayu',
        'pl': 'Poland',
        'pt_BR': 'Portugis (Brazil)',
        'ru': 'Rusia',
        'uk': 'Ukraine',
        'uz': 'Uzbekistan',
        'zh_TW': 'Cina (Taiwan)'},
 'pl': {'ast': 'asturyjski',
        'bg': 'bułgarski',
        'bs': 'bośniacki',
        'cs': 'czeski',
        'de': 'niemiecki',
        'el': 'grecki',
        'en_GB': 'angielski (Wielka Brytania)',
        'es': 'hiszpański',
        'fi': 'fiński',
        'fr': 'francuski',
        'gl': 'galicyjski',
        'he': 'hebrajski',
        'it': 'włoski',
        'kab': 'kabylski',
        'ms': 'malajski',
        'pl': 'polski',
        'pt_BR': 'portugalski (Brazylia)',
        'ru': 'rosyjski',
        'uk': 'ukraiński',
        'uz': 'uzbecki',
        'zh_TW': 'chiński (Tajwan)'},
 'pt_BR': {'ast': 'asturiano',
           'bg': 'búlgaro',
           'bs': 'bósnio',
           'cs': 'tcheco',
           'de': 'alemão',
           'el': 'grego',
           'en_GB': 'inglês (Reino Unido)',
           'es': 'espanhol',
           'fi': 'finlandês',
           'fr': 'francês',
           'gl': 'galego',
           'he': 'hebraico',
           'it': 'italiano',
           'kab': 'kabyle',
           'ms': 'malaio',
           'pl': 'polonês',
           'pt_BR': 'português (Brasil)',
           'ru': 'russo',
           'uk': 'ucraniano',
           'uz': 'usbeque',
           'zh_TW': 'chinês (Taiwan)'},
 'ru': {'ast': 'астурийский',
        'bg': 'болгарский',
        'bs': 'боснийский',
        'cs': 'чешский',
        'de': 'немецкий',
        'el': 'греческий',
        'en_GB': 'английский (Великобритания)',
        'es': 'испанский',
        'fi': 'финский',
        'fr': 'французский',
        'gl': 'галисийский',
        'he': 'иврит',
        'it': 'итальянский',
        'kab': 'кабильский',
        'ms': 'малайский',
        'pl': 'польский',
        'pt_BR': 'португальский (Бразилия)',
        'ru': 'русский',
        'uk': 'украинский',
        'uz': 'узбекский',
        'zh_TW': 'китайский (Тайвань)'},
 'uk': {'ast': 'астурська',
        'bg': 'болгарська',
        'bs': 'боснійська',
        'cs': 'чеська',
        'de': 'німецька',
        'el': 'грецька',
        'en_GB': 'англійська (Велика Британія)',
        'es': 'іспанська',
        'fi': 'фінська',
        'fr': 'французька',
        'gl': 'галісійська',
        'he': 'іврит',
        'it': 'італійська',
        'kab': 'кабільська',
        'ms': 'малайська',
        'pl': 'польська',
        'pt_BR': 'португальська (Бразилія)',
        'ru': 'російська',
        'uk': 'українська',
        'uz': 'узбецька',
        'zh_TW': 'китайська (Тайвань)'},
 'uz': {'bg': 'bolgar',
        'bs': 'bosniy',
        'cs': 'chex',
        'de': 'nemischa',
        'el': 'grek',
        'en_GB': 'inglizcha (Birlashgan Qirollik)',
        'es': 'ispancha',
        'fi': 'fincha',
        'fr': 'fransuzcha',
        'gl': 'galisiy',
        'he': 'ibroniy',
        'it': 'italyan',
        'kab': 'kabil',
        'ms': 'malay',
        'pl': 'polyakcha',
        'pt_BR': 'portugalcha (Braziliya)',
        'ru': 'ruscha',
        'uk': 'ukrain',
        'uz': 'o‘zbek',
        'zh_TW': 'xitoy (Tayvan)'},
 'zh_TW': {'ast': '阿斯圖里亞文',
           'bg': '保加利亞文',
           'bs': '波士尼亞文',
           'cs': '捷克文',
           'de': '德文',
           'el': '希臘文',
           'en_GB': '英文（英國）',
           'es': '西班牙文',
           'fi': '芬蘭文',
           'fr': '法文',
           'gl': '加利西亞文',
           'he': '希伯來文',
           'it': '義大利文',
           'kab': '卡比爾文',
           'ms': '馬來文',
           'pl': '波蘭文',
           'pt_BR': '葡萄牙文（巴西）',
           'ru': '俄文',
           'uk': '烏克蘭文',
           'uz': '烏茲別克文',
           'zh_TW': '中文（台灣）'}}
)

